@@ -8,7 +8,7 @@ package Net::POP3S;
 
 use vars qw ( $VERSION @ISA );
 
-$VERSION = "0.05";
+$VERSION = "0.06";
 
 use base qw ( Net::POP3 );
 use Net::Cmd;  # import CMD_OK, CMD_MORE, ...
@@ -38,6 +38,7 @@ sub new {
       $host = delete $arg{Host};
   }
   my $ssl = delete $arg{doSSL};
+  $ssl = 'ssl' if delete $arg{SSL};
 
   my $hosts = defined $host ? $host : $NetConfig{pop3_hosts};
   my $obj;
@@ -131,122 +132,39 @@ sub ssl_start {
     ) or return undef;
 }
 
+
+sub capa {
+    my $this = shift;
+
+    if (exists ${*$this}{'net_pop3e_capabilities'}) {
+	return ${*$this}{'net_pop3e_capabilities'};
+    }
+    $this->SUPER::capa();
+}
+
 # Override to specify a certain auth mechanism.
 sub auth {
-  my ($self, $username, $password, $mechs) = @_;
+  my ($self, $username, $password, $mech) = @_;
 
-  eval {
-    require MIME::Base64;
-    require Authen::SASL;
-  } or $self->set_status(500, ["Need MIME::Base64 and Authen::SASL todo auth"]), return 0;
+  if ($mech) {
+      $self->debug_print(1, "my favorite: ". $mech . "\n") if $self->debug;
 
-  my $mechanisms = undef;
-  if ($mechs) {
-      $mechanisms = $mechs;
-  } else {
-    my $capa    = $self->capa;
-    $mechanisms = $capa->{SASL} || 'CRAM-MD5';
-  }
+      my @cl_mech = split /\s+/, $mech;
+      my @matched = ();
+      my $sv = $self->capa->{SASL} || 'CRAM-MD5';
 
-  my $sasl;
-
-  if (ref($username) and UNIVERSAL::isa($username, 'Authen::SASL')) {
-    $sasl = $username;
-    my $user_mech = $sasl->mechanism || '';
-    my @user_mech = split(/\s+/, $user_mech);
-    my %user_mech;
-    @user_mech{@user_mech} = ();
-
-    my @server_mech = split(/\s+/, $mechanisms);
-    my @mech = @user_mech
-      ? grep { exists $user_mech{$_} } @server_mech
-      : @server_mech;
-    unless (@mech) {
-      $self->set_status(
-        500,
-        [ 'Client SASL mechanisms (',
-          join(', ', @user_mech),
-          ') do not match the SASL mechnism the server announces (',
-          join(', ', @server_mech), ')',
-        ]
-      );
-      return 0;
-    }
-
-    $sasl->mechanism(join(" ", @mech));
-
-    $sasl->mechanism($mechanisms);
-  }
-  else {
-    die "auth(username, password)" if not length $username;
-    $sasl = Authen::SASL->new(
-      mechanism => $mechanisms,
-      callback  => {
-        user     => $username,
-        pass     => $password,
-        authname => $username,
+      foreach my $i (@cl_mech) {
+	  if (index($sv, $i) >= 0 && grep(/$i/i, @matched) == () ) {
+	      push @matched, uc($i);
+	  }
+      }
+      if (@matched) {
+      ## override AUTH mech as specified.
+      ## if multiple mechs are specified, priority is still up to Authen::SASL module.
+	  ${*$self}{'net_pop3e_capabilities'}->{'SASL'} = join " ", @matched;
       }
-    );
   }
-
-  # We should probably allow the user to pass the host, but I don't
-  # currently know and SASL mechanisms that are used by smtp that need it
-  my ($hostname) = split /:/, ${*$self}{'net_pop3_host'}; # /
-  my $client = eval { $sasl->client_new('pop', $hostname, 0) };
-
-  unless ($client) {
-    my $mech = $sasl->mechanism;
-    $self->set_status(
-      500,
-      [ " Authen::SASL failure: $@",
-        '(please check if your local Authen::SASL installation',
-        "supports mechanism '$mech'"
-      ]
-    );
-    return 0;
-  }
-
-  my ($token) = $client->client_start
-    or do {
-    my $mech = $client->mechanism;
-    $self->set_status(
-      500,
-      [ ' Authen::SASL failure:  $client->client_start ',
-        "mechanism '$mech' hostname #$hostname#",
-        $client->error
-      ]
-    );
-    return 0;
-    };
-
-  # We dont support sasl mechanisms that encrypt the socket traffic.
-  # todo that we would really need to change the ISA hierarchy
-  # so we dont inherit from IO::Socket, but instead hold it in an attribute
-
-  my @cmd = ("AUTH", $client->mechanism);
-  my $code;
-
-  push @cmd, MIME::Base64::encode_base64($token, '')
-    if defined $token and length $token;
-
-  while (($code = $self->command(@cmd)->response()) == CMD_MORE) {
-
-    my ($token) = $client->client_step(MIME::Base64::decode_base64(($self->message)[0])) or do {
-      $self->set_status(
-        500,
-        [ ' Authen::SASL failure:  $client->client_step ',
-          "mechanism '", $client->mechanism, " hostname #$hostname#, ",
-          $client->error
-        ]
-      );
-      return 0;
-    };
-
-    @cmd = (MIME::Base64::encode_base64(defined $token ? $token : '', ''));
-  }
-
-  $code == CMD_OK;
-
+  $self->SUPER::auth($username, $password);
 }
 
 1;
@@ -304,7 +222,7 @@ Tomo.M E<lt>tomo at cpan orgE<gt>
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (C) 2013 by Tomo
+Copyright (C) 2017 by Tomo.M
 
 This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself, either Perl version 5.8.8 or,
